/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include <dlfcn.h>

#include "plugin_manager.h"
#include "utils.h"
#include "log.h"

static PluginFunc* __pluginFunc = NULL;
static void* __pluginLib;

#define PLUGIN_PATH "/usr/share/dotnet.tizen/lib/libdotnet_plugin.so"

int initializePluginManager(const char* mode)
{
	if (isFileExist(PLUGIN_PATH)) {
		__pluginLib = dlopen(PLUGIN_PATH, RTLD_NOW | RTLD_LOCAL);
		if (__pluginLib) {
			__pluginFunc = (PluginFunc*)calloc(sizeof(PluginFunc), 1);
			if (!__pluginFunc) {
				_ERR("fail to allocate memory for plugin function structure");
				return -1;
			}
			__pluginFunc->initialize = (plugin_initialize_ptr)dlsym(__pluginLib, "plugin_initialize");
			__pluginFunc->preload = (plugin_preload_ptr)dlsym(__pluginLib, "plugin_preload");
			__pluginFunc->has_log_control = (plugin_has_log_control_ptr)dlsym(__pluginLib, "plugin_has_log_control");
			__pluginFunc->set_app_info = (plugin_set_app_info_ptr)dlsym(__pluginLib, "plugin_set_app_info");
			__pluginFunc->set_coreclr_info = (plugin_set_coreclr_info_ptr)dlsym(__pluginLib, "plugin_set_coreclr_info");
			__pluginFunc->get_dll_path = (plugin_get_dll_path_ptr)dlsym(__pluginLib, "plugin_get_dll_path");
			__pluginFunc->before_execute = (plugin_before_execute_ptr)dlsym(__pluginLib, "plugin_before_execute");
			__pluginFunc->finalize  = (plugin_finalize_ptr)dlsym(__pluginLib, "plugin_finalize");
		}

		if (__pluginFunc->initialize)
			__pluginFunc->initialize(mode);
	}

	_INFO("Plugin manager initialize success");

	return 0;
}

void finalizePluginManager()
{
	if (__pluginFunc) {
		if (__pluginFunc->finalize) {
			__pluginFunc->finalize();
		}
		free(__pluginFunc);
		__pluginFunc = NULL;
	}

	if (__pluginLib) {
		dlclose(__pluginLib);
		__pluginLib = NULL;
	}
}

void pluginPreload()
{
	if (__pluginFunc && __pluginFunc->preload) {
		__pluginFunc->preload();
	}
}

bool pluginHasLogControl()
{
	if (__pluginFunc && __pluginFunc->has_log_control) {
		return __pluginFunc->has_log_control();
	} else {
		return false;
	}
}

void pluginSetAppInfo(const char* appId, const char* managedAssemblyPath)
{
	if (__pluginFunc && __pluginFunc->set_app_info) {
		__pluginFunc->set_app_info(appId, managedAssemblyPath);
	}
}

void pluginSetCoreclrInfo(void* hostHandle, unsigned int domainId, coreclr_create_delegate_ptr delegateFunc)
{
	if (__pluginFunc && __pluginFunc->set_coreclr_info) {
		__pluginFunc->set_coreclr_info(hostHandle, domainId, delegateFunc);
	}
}

char* pluginGetDllPath()
{
	if (__pluginFunc && __pluginFunc->get_dll_path) {
		return __pluginFunc->get_dll_path();
	} else {
		return NULL;
	}
}

void pluginBeforeExecute()
{
	if (__pluginFunc && __pluginFunc->before_execute) {
		__pluginFunc->before_execute();
	}
}

